@@ -1,6 +1,5 @@
 #!/usr/bin/env python3
 
-# from werkzeug.serving import WSGIRequestHandler
 from logging import error, exception
 from socket import (
     AF_INET,
@@ -11,15 +10,25 @@ from socket import (
     TCP_KEEPCNT,
     TCP_KEEPIDLE,
     TCP_KEEPINTVL,
+    getaddrinfo,
     socket,
 )
+from threading import Timer
 from time import sleep
 from wsgiref.simple_server import WSGIRequestHandler
 
+from .resource_balancer import ResourceBalancer
+
 
 class Handler(WSGIRequestHandler):
     protocol_version = "HTTP/1.1"
 
+    def handle_one_request(self):
+        try:
+            super().handle_one_request()
+        finally:
+            close_connection = True
+
 
 class Server:
     ssl_context = None
@@ -30,7 +39,7 @@ class Server:
     shutdown_signal = False
 
     def __init__(self, addr, app):
-        (host, port) = self.addr = addr
+        (host, port) = addr
         # Set up base environment
         env = self.base_environ = {}
         env["SERVER_NAME"] = host
@@ -40,13 +49,16 @@ class Server:
         env["CONTENT_LENGTH"] = ""
         env["SCRIPT_NAME"] = ""
 
+        self.balancer = LoadBalancer(host, port)
         self.app = app
 
     def run(self):
         while True:
-            with socket(AF_INET, SOCK_STREAM) as s:
+            with socket(
+                AF_INET, SOCK_STREAM
+            ) as s, self.balancer.acquire_addr() as addr:
                 try:
-                    s.connect(self.addr)
+                    s.connect(addr)
                 except Exception as e:
                     error("Connect: %s", e)
                     sleep(2)
@@ -58,8 +70,8 @@ class Server:
                 s.setsockopt(IPPROTO_TCP, TCP_KEEPCNT, 3)
 
                 try:
-                    Handler(s, self.addr, server)
-                except Exception:
+                    Handler(s, addr, server)
+                except:
                     exception("WSGIRequestHandler")
                     sleep(1)
                     continue
@@ -68,6 +80,29 @@ class Server:
         return self.app
 
 
+class LoadBalancer:
+    REFRESH_INTERVAL = 5
+
+    def __init__(self, host, port):
+        self.host = host
+        self.port = port
+        self._balancer = ResourceBalancer()
+        self._timer = Timer(self.REFRESH_INTERVAL, self.refresh)
+        self.refresh()
+        self._timer.start()
+
+    def acquire_addr(self):
+        return self._balancer.acquire()
+
+    def refresh(self):
+        try:
+            results = getaddrinfo(self.host, self.port, type=SOCK_STREAM)
+            hosts = (sockaddr for _f, _t, _p, _c, sockaddr in results)
+            self._balancer.provision(hosts)
+        except:
+            exception("Failed to refresh endpoints for %s:%d", self.host, self.port)
+
+
 if __name__ == "__main__":
     from argparse import ArgumentParser
     from importlib import import_module
